#!/usr/bin/perl

# delexpired.pl
#
# by M.D. Brownsworth
# 
# Version 2.2
# 17 Apr 01
#
# Companion crontab script to popauthd SMTP authorization daemon.
# Deletes expired entries from sendmail access files, rehashes db.
#
# Example entry from access.info file:
# Mar 20 14:03:01    michelle    24.14.231.140   985125786
#
# Example crontab:
# Delete expired entries in relay allow file at 15 minutes after hour
# 15  *    *    *    *    root  /etc/cron_scripts/delexpired.pl

$maildir = "/etc/mail";
$access = "$maildir/access";
$makeaccess ="$maildir/POPauthd/makeaccess";
$newtimestamp = time;

# Uncomment desired expiration value below
#$expires = 3600; # 1 hour
#$expires = 10800; # 3 hours
#$expires = 21600; # 6 hours
#$expires = 43200; # 12 hours
$expires = 86400; # 1 day
#$expires = 129600; # 3 days
#$expires = 604800; # 1 week
#$expires = 1209600; # 2 weeks
#$expires = 1814400; # 3 weeks
#$expires = 2419200; # 1 month

open(INFO, "${access}.info") || die "Can't open ${access}.info: $!\n";
open(TEMP, ">>${access}.temp") || die "Can't open ${access}.temp: $!\n";
while(<INFO>) {
	($date,$userid,$ip,$timestamp) = split(/\t/, $_);
	chomp($timestamp); # Trim trailing newline char
	next if (($newtimestamp - $timestamp) > $expires);
	print TEMP;
}
close(TEMP);
close(INFO);

rename("${access}.temp","${access}.info");
system("awk -F\\\t '{ print \$3 \"\tRELAY\" }' ${access}.info > ${access}.relay");
system("[ ! -r ${access}.reject ] && { touch ${access}.reject; }"); # Create reject file if none exists
system("cat ${access}.reject ${access}.relay > $access"); # Combine reject and relay files
system("$makeaccess"); # Rehash access.db

exit(0);
