#!/usr/bin/perl
# Script for uninstalling FreeVSD following compilation from source.
# (RedHat GNU Linux version)
# Copyright (c) 1999, 2000 Idaya Ltd.
#
# This file is part of the Virtual Server Administrator (FreeVSD)
#
# FreeVSD is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# FreeVSD is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with FreeVSD; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#

# use strict; # FIX and -w -- Still not ready for this yet until VSD.pm
#                              is strict too and everything else...

use VSD;
use VSD::Undo;
use VSD::Config;

my $vsdconfdir = "$path{vsdconfdir}";
my $vsdcertsdir = "$vsdconfdir/certs";
my $vsdkeysdir = "$vsdconfdir/private";
my $vsdcadir = "$vsdconfdir/ca";
my $vsdundodir = "$vsdconfdir/undo";


my $vsdconf = VSD::Config->new( file=>$path{vsdconf} );


my $localhost = `hostname`;
chomp $localhost;
$localhost =~ s/^\s*(\S+)\s*$/$1/;


#my $mntpoint = $vsdconf->getval(
#				'Partition',
#				'0',
#				'Mount'
#			       );
#  FIX soon not only Partition 0



my $vsd_install_undo = VSD::Undo->new( script => 'vsd-install.pl' );

defined $vsd_install_undo or die "Nothing to uninstall.";




# Turn off virtual servers..
system "$path{vsdsbindir}/vsboot --stop";




my %files = %{$vsd_install_undo->file_list()};
my %commands = %{$vsd_install_undo->command_list()};
my %command_comments = %{$vsd_install_undo->command_comments()};

foreach my $file ( keys %files ) {
    print "\nUndo changes to $file ? [Y] : ";
    if (confirmy()) {
	$vsd_install_undo->file( $file );
    }
}

# FIX needs ordering...

foreach my $command ( keys %commands ) {
    print "\n".(defined $command_comments{$command} ? 
		"Do you want to $command_comments{$command}" :
		"Undo command $command").' ? [Y] : ';
    if (confirmy()) {
	print "  Executing command: $commands{$command} ...\n";
	$vsd_install_undo->command( $command );
    }
}




# ------------------- vsd-genskel changes ------------------------------

# FIX new path stuff ..... ?
if (-d "$vsdcadir") {
    print "\n  Remove VSD Certificate Authority? [N] : ";
    if (confirmn()) {
	system ("rm", "-rf", "$vsdcadir");
    }
}

if (-d "$vsdcertsdir") {
    print "\n  Remove issued VSD certificates and keys beneath $vsdconfdir? [N] : ";
    if (confirmn()) {
	system ("rm", "-rf", "$vsdcertsdir");
	system ("rm", "-rf", "$vsdkeysdir");
    }
}

if (-d "$vsdundodir") {
    print "\n  Remove vsd undo directory and log, $vsdundodir? [Y] : ";
    if (confirmy()) {
	system ("rm", "-rf", "$vsdundodir");
    }
}

my $remove_all = 0;
if (-d "$vsdconfdir") {
    print "\n  Remove all VSD configuration, $vsdconfdir? [N] : ";
    if (confirmn()) {
	$remove_all = 1;
    }
}

my @dead_dirs;
my %partitions = $vsdconf->getval( 'Partition' );

foreach my $partition ( keys %partitions ) {
    
    print "\nPartition $partition ...";
    
    my $mount = $vsdconf->getval(
				 'Partition',
				 $partition,
				 'Mount'
				 );
    my @skels = $vsdconf->getval(
				 'Partition',
				 $partition,
				 'Skel'
				 );
    
    my $rm_count = 0;
    foreach my $skel ( @skels ) {
	defined $skel || next;
	if (-d "$mount/skel/$skel" && $skel ne '') {
	    print "\n  Remove skel, $skel ? [N] : ";
	    confirmn() && do {
		push (@dead_dirs, "$mount/skel/$skel");
		$rm_count++;
	    };
	}
    }
    
    if ( -d "$mount" ) {
	if ( $rm_count >= @skels ) {
	    print "\n    All skels will be removed...\n";
	    print "\n  Remove mount point, $mount, ";
	    print "including all virtual servers beneath? [N] : ";
	    if (confirmn()) {
		push (@dead_dirs, "$mount");
	    }
	}
	else {
	    print "\n  Remove virtual servers beneath mount point, $mount ? [N] : ";
	    if (confirmn()) {
		push (@dead_dirs, "$mount/vs");
	    }	
	}		
    }
}

print "\n";
foreach (@dead_dirs) {
    print "    Deleting $_...\n";
    system ("rm", "-rf", "$_");
 }

if ($remove_all == 1) {
    system ("rm", "-rf", "$vsdconfdir");
}

print "\n...finished.\n";


sub confirmy {
    $_ = <>;
    return 0 if (/^n.*/i);
    return 1;
}

sub confirmn {
    $_ = <>;
    return 1 if (/^y.*/i);
    return 0;
}



