/* Timezone module command interface.
   Copyright (c) 2001 Designer Servers Ltd.  All rights reserved.
   Written by Nick Burrett <nick@dsvr.net>.  */

#include <stdio.h>
#include "vsd.h"
#include "module.h"
#include "mod_tz.h"

static struct command_tag clist[] =
{
  {
    CMD_TZ_LIST, ADMIN_VS,
    "List available timezones",
    "TZ_LIST", 0, "",
    "tz_list", 2, "<host svr> <vs>"
  },
  {
    CMD_TZ_CHANGE, ADMIN_VS,
    "Change timezone for a virtual server",
    "TZ_CHANGE", 1, "<country>",
    "tz_change", 3, "<host svr> <vs> <country>"
  },
  {
    0, 0, NULL, NULL, 0, NULL, NULL, 0, NULL
  }
};

/* Return -1 if `cmd' does not exist.
   Return 1 if there is a syntax error.
   Return 0 if everything is cool.  */
int mod_tz_check_cmd (int level, const char *cmd,
		      int argc, char *argv[], struct command_tag **list)
{
  if (cmd == NULL)
    {
      /* Return a pointer to all commands.  */
      *list = clist;
      return 0;
    }

  /* Find command in clist.  */
  *list = module_lookup_command (clist, level, cmd);
  if (*list == NULL)
    return -1;  /* Command does not exist.  */

  /* Check command syntax.  Pretty basic at the moment.  */
#ifdef VSD_LIB
  if (argc < (*list)->vsdadm_args)
    return 1; /* Syntax error.  */
#else
  if (argc < (*list)->vsd_args)
    return 1; /* Syntax error.  */
#endif

  /* Syntax is ok.  */
  return 0;
}

#ifdef VSD_LIB
char *mod_tz_cmd (void *io, int level, const char *cmd,
		  int argc, char **argv)
{
  char *status = NULL;
  struct command_tag *list = module_lookup_command (clist, level, cmd);
  if (! list)
    return "notexist";

  vsd_set_level (io, list->level);
  switch (list->token)
    {
    case CMD_TZ_LIST:
      vsd_send_command (io, NULL, "TZ_LIST");
      break;
    case CMD_TZ_CHANGE:
      vsd_send_command (io, NULL, "TZ_CHANGE %s", argv[2] /* country */);
      break;
    default:
      return "mod_tz: command not recognised";
      break;
    }

  status = vsd_process_reply (io);
  return status;
}

#else /* ! VSD_LIB */

/* Return 0 if command processed, 1 if not recognised.  */
int mod_tz_cmd (struct connection *vc,
		int level, const char *command,
		int argc, char *argv[], const char *data)
{
  int token;
  struct command_tag *tag = module_lookup_command (clist, level, command);

  if (tag == NULL)
    return 1;

  token = tag->token;
  switch (token)
    {
    case CMD_TZ_LIST:
      tz_list (vc, argc, argv);
      break;
    case CMD_TZ_CHANGE:
      tz_change (vc, argc, argv);
      break;
    default:
      return 1;
      break;
    }

  return 0;
}

#endif

module tz_module =
{
  mod_tz_check_cmd,
  mod_tz_cmd
};
