/* vd.h - vd module global definitions
 */

/*
 * VSD - Virtual Server Daemon
 * Copyright (c) 2000 - 2001 Idaya Ltd.
 *
 * VSD is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * VSD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VSD - see the COPYING file;  if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * See the AUTHORS file for a list of contributors to VSD.
 * See the ChangeLog files for a list of changes.
 */

#ifndef VD_H
#define VD_H

extern void 
vd_account_create (struct connection *vc, 
                   int argc, 
                   char *argv[]);

extern void 
vd_account_delete (struct connection *vc, 
                   int argc, 
                   char *argv[]);

extern void
vd_alias_add  (struct connection *vc,
               int argc,
               char *argv[]);

extern void
vd_alias_del  (struct connection *vc,
                 int argc,
                 char *argv[]);


extern void 
vd_details (struct connection *vc, 
            int level, 
            int argc, 
            char *argv[]);

extern void 
vd_modify (struct connection *vc, 
           int argc, 
           char *argv[]);

const char *
vd_valid_domainname (const char *vd);

int 
vd_map_add_vdalias (struct vsd_vs_map *map,
                        const char *virtual_server,
                        const char *ipalias);

int 
vd_map_delete_vdalias (struct vsd_vs_map *map,
                           const char *virtual_server,
                           const char *ipalias);

static char *
vd_procmailrc_path (const char *root,
                    const char *domain);

static char *
vd_passwd_path (char *root,
                char *domain);

static char *
vd_spool_path (char *root,
               char *domain);

static char *
vd_passwd_string_create (const char *passwd_string);

static char *
vd_passwd_file_create (const char *user,
                       const char *domain_name,
                       const char *path_to_passwd_file);

int
vd_chown_run_command (struct connection *vc,
                      const char *hosting_server,
                      char *user,
                      char *file_to_change);

static char *
vd_mkdir (const char *root_dir,
          const char *dir_to_make);

int
vd_refresh_mailertable (struct connection *vc,
                        const char *domain_root_path,
                        const char *database_file, /* mailtertable.db file */
                        const char *input_file /* mailtable file */);

int
vd_append_domain_mailertable (const char *domain_root_path,
                              const char *domain_identifier,
                              const char *domain_name);

static char *
vd_remove_domain_mailertable (const char *domain_root_path,
                              const char *domain_name);

static char *
vd_config_domain (struct connection *vc,
                  struct vsd_vs *domain);

static char *
vd_remove_domain  (struct connection *vc,
                   struct vsd_vs *domain);

static char *
vd_config_domain_alias (struct connection *vc,
                        struct vsd_vs *domain,
                        const char *domain_alias);

static char *
vd_remove_domain_alias (struct connection *vc,
                        struct vsd_vs *domain,
                        const char *domain_alias);

static char *
vd_dir_files_link (const char *dir_from,
                   const char *dir_to);

static char *
vd_dir_files_unlink (const char *dir);

static char *
vd_check_domain_limit (struct vsd_vs_map *map,
                      struct vsd_vs *virtual_server,
                      const char *root_path);

#endif /* VD_H */
