/* mod_vuser.c - vuser module interface.
 */

/* 
 * VSD - Virtual Server Daemon
 * Copyright (c) 2000 - 2001 Idaya Ltd.
 *
 * VSD is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later version.
 *
 * VSD is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with VSD - see the COPYING file;  if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.  
 */

/*
 * See the AUTHORS file for a list of contributors to VSD. 
 * See the ChangeLog files for a list of changes. 
 */

#include <stdio.h>

#include "module.h"
#include "mod_vuser.h"
#include "vuser.h"

static struct command_tag clist[] =
{
  {
    CMD_VUSER_ADD, ADMIN_VD,
    "Add virtual user",
    "VUSER_ADD", 1, "<name>",
    "vuser_add", 3, "<hs> <domain> <name>"
  },
  {
    CMD_VUSER_DEL, ADMIN_VD,
    "Delete virtual user",
    "VUSER_DEL", 1, "<name>",
    "vuser_del", 3, "<hs> <domain> <name>"
  },
  {
    CMD_VUSER_DFLTADD, ADMIN_VD,
    "Add virtual user default forwarder",
    "VUSER_DFLTADD", 2, "<name> <fwd>",
    "vuser_dfltadd", 4, "<hs> <domain> <name> <fwd>"
  },
  {
    CMD_VUSER_DFLTDEL, ADMIN_VD,
    "Delete virtual user default forwarder",
    "VUSER_DFLTDEL", 2, "<name> <fwd>",
    "vuser_dfltdel", 4, "<hs> <domain> <name> <fwd>"
  },
  {
    CMD_VUSER_FWDADD, ADMIN_VD,
    "Add forwarder to virtual user",
    "VUSER_FWDADD", 2, "<name> <fwd>",
    "vuser_fwdadd", 4, "<hs> <domain> <name> <fwd>"
  },
  {
    CMD_VUSER_FWDDEL, ADMIN_VD,
    "Delete forwarder from virtual user",
    "VUSER_FWDDEL", 2, "<name> <fwd>",
    "vuser_fwddel", 4, "<hs> <domain> <name> <fwd>"
  },
  {
    CMD_VUSER_RSPADD, ADMIN_VD,
    "Add responder to virtual user",
    "VUSER_RSPADD", 2, "<name> <msg>",
    "vuser_rspadd", 4, "<hs> <domain> <name> <rsp>"
  },
  {
    CMD_VUSER_RSPDEL, ADMIN_VD,
    "Delete responder from virtual user",
    "VUSER_RSPDEL", 2, "<name> <msg>",
    "vuser_rspdel", 4, "<hs> <domain> <name> <rsp>"
  },
  {
    CMD_VUSER_LIST, ADMIN_VD,
    "List virtual users",
    "VUSER_LIST", 0, "[<vuser>]",
    "vuser_list", 2, "<hs> <domain> [<vuser>]"
  },
  {
    CMD_VUSER_MBXADD, ADMIN_VD,
    "Add virtual user mailbox",
    "VUSER_MBXADD", 1, "<name>",
    "vuser_mbxadd", 3, "<hs> <domain> <name> [<passwd>]"
  },
  {
    CMD_VUSER_MBXDEL, ADMIN_VD,
    "Delete virtual user mailbox",
    "VUSER_MBXDEL", 1, "<name>",
    "vuser_mbxdel", 3, "<hs> <domain> <name>"
  },
  {
    CMD_VUSER_MBXLIST, ADMIN_VD,
    "List virtual user mailboxes",
    "VUSER_MBXLIST", 0, "",
    "vuser_mbxlist", 2, "<hs> <domain>"
  },
  {
    CMD_VUSER_MBXPASS, ADMIN_VD,
    "Change virtual user mailbox password",
    "VUSER_MBXPASS", 3, "<name> <old> <new>",
    "vuser_mbxpass", 5, "<hs> <domain> <name> <old> <new>"
  },
  {
    CMD_VUSER_MBXENABLE, ADMIN_VD,
    "Enable virtual user mailbox",
    "VUSER_MBXENABLE", 1, "<name>",
    "vuser_mbxenable", 3, "<hs> <domain> <name>"
  },
  {
    CMD_VUSER_MBXDISABLE, ADMIN_VD,
    "Disable virtual user mailbox",
    "VUSER_MBXDISABLE", 1, "<name>",
    "vuser_mbxdisable", 3, "<hs> <domain> <name>"
  },
  {
    CMD_VUSER_MSGADD, ADMIN_VD,
    "Add virtual user response message",
    "VUSER_MSGADD", 2, "<name> <msg> <file>",
    "vuser_msgadd", 4, "<hs> <domain> <name> <msg>"
  },
  {
    CMD_VUSER_MSGDEL, ADMIN_VD,
    "Delete virtual user response message",
    "VUSER_MSGDEL", 2, "<name> <msg>",
    "vuser_msgdel", 4, "<hs> <domain> <name> <msg>"
  },
  {
    CMD_VUSER_MSGGET, ADMIN_VD,
    "Get virtual user response message",
    "VUSER_MSGGET", 2, "<name> <msg>",
    "vuser_msgget", 4, "<hs> <domain> <name> <msg>"
  },
  {
    CMD_VUSER_MSGSET, ADMIN_VD,
    "Set virtual user response message",
    "VUSER_MSGSET", 2, "<name> <msg>",
    "vuser_msgset", 5, "<hs> <domain> <name> <msg> <file>"
  },
  {
    CMD_VUSER_MSGLIST, ADMIN_VD,
    "List virtual user response messages",
    "VUSER_MSGLIST", 1, "<name>",
    "vuser_msglist", 3, "<hs> <domain> <name>"
  },
  {
    0, 0, NULL, NULL, 0, NULL, NULL, 0, NULL
  }
};

/* Return -1 if `cmd' does not exist.
   Return 1 if there is a syntax error.
   Return 0 if everything is cool.  */
int mod_vuser_check_cmd (int level, const char *cmd, 
			 int argc, char *argv[],
			 struct command_tag **list)
{
  if (cmd == NULL)
    {
      /* Return a pointer to all commands.  */
      *list = clist;
      return 0;
    }

  /* Find command in clist.  */
  *list = module_lookup_command (clist, level, cmd);
  if (*list == NULL)
    return -1;  /* Command does not exist.  */

  /* Check command syntax.  Pretty basic at the moment.  */
#ifdef VSD_LIB
  if (argc < (*list)->vsdadm_args)
    return 1; /* Syntax error.  */
#else
  if (argc < (*list)->vsd_args)
    return 1; /* Syntax error.  */
#endif

  /* Syntax is ok.  */
  return 0;
}

#ifdef VSD_LIB
char *mod_vuser_cmd (void *io, int level, const char *cmd,
		     const int argc, const char **argv)
{
  struct command_tag *list;
  char *status = NULL;

  list = module_lookup_command (clist, level, cmd);
  if (! list)
    return "notexist";

  vsd_set_level (io, list->level);
  switch (list->token)
    {
    case CMD_VUSER_ADD:
	vsd_send_command (io, NULL, 
			  "VUSER_ADD %s",
			  argv[2]);   /* name */
      status = vsd_process_reply (io);
      break;
    case CMD_VUSER_DEL:
      vsd_send_command (io, NULL, 
			"VUSER_DEL %s",
			argv[2]);  /* name */
      status = vsd_process_reply (io);
      break;
    case CMD_VUSER_DFLTADD:
	vsd_send_command (io, NULL, 
			  "VUSER_DFLTADD %s %s",
			  argv[2],   /* name */
			  argv[3]);  /* fwd */
      status = vsd_process_reply (io);
      break;
    case CMD_VUSER_DFLTDEL:
      vsd_send_command (io, NULL, 
			"VUSER_DFLTDEL %s %s",
			argv[2],   /* name */
			argv[3]);  /* fwd */
      status = vsd_process_reply (io);
      break;
    case CMD_VUSER_FWDADD:
      vsd_send_command (io, NULL, 
			"VUSER_FWDADD %s %s",
			argv[2],   /* name */
			argv[3]);  /* fwd */
      status = vsd_process_reply (io);
      break;
    case CMD_VUSER_FWDDEL:
      vsd_send_command (io, NULL, 
			"VUSER_FWDDEL %s %s",
			argv[2],   /* name */
			argv[3]);  /* fwd */ 
      status = vsd_process_reply (io);
      break;
    case CMD_VUSER_RSPADD:
      vsd_send_command (io, NULL, 
			"VUSER_RSPADD %s %s",
			argv[2],   /* name */
			argv[3]);  /* rsp */
      status = vsd_process_reply (io);
      break;
    case CMD_VUSER_RSPDEL:
      vsd_send_command (io, NULL, 
			"VUSER_RSPDEL %s %s",
			argv[2],   /* name */
			argv[3]);  /* rsp */ 
      status = vsd_process_reply (io);
      break;
    case CMD_VUSER_LIST:
      if (argv[2])
	vsd_send_command (io, NULL, 
			  "VUSER_LIST %s",
			  argv[2]);  /* name */
      else
	vsd_send_command (io, NULL, 
			  "VUSER_LIST");
      status = vsd_process_reply (io);
      break;
    case CMD_VUSER_MBXADD:
      if (argv[3])
	vsd_send_command (io, NULL, 
			  "VUSER_MBXADD %s %s",
			  argv[2],   /* name */
			  argv[3]);  /* passwd */
      else
	vsd_send_command (io, NULL, 
			  "VUSER_MBXADD %s",
			  argv[2]);  /* name */
      status = vsd_process_reply (io);
      break;
    case CMD_VUSER_MBXDEL:
      vsd_send_command (io, NULL, 
			"VUSER_MBXDEL %s",
			argv[2]);  /* name */
      status = vsd_process_reply (io);
      break;
    case CMD_VUSER_MBXLIST:
      vsd_send_command (io, NULL, 
			"VUSER_MBXLIST");
      status = vsd_process_reply (io);
      break;
    case CMD_VUSER_MBXPASS:
      vsd_send_command (io, NULL, 
			"VUSER_MBXPASS %s %s %s",
			argv[2],   /* name */
			argv[3],   /* old */
			argv[4]);  /* new */
      status = vsd_process_reply (io);
      break;
    case CMD_VUSER_MBXENABLE:
      vsd_send_command (io, NULL, 
			"VUSER_MBXENABLE %s",
			argv[2]);  /* name */
      status = vsd_process_reply (io);
      break;
    case CMD_VUSER_MBXDISABLE:
      vsd_send_command (io, NULL, 
			"VUSER_MBXDISABLE %s",
			argv[2]);  /* name */
      status = vsd_process_reply (io);
      break;
    case CMD_VUSER_MSGADD:
      vsd_send_command (io, NULL, 
			"VUSER_MSGADD %s %s",
			argv[2],   /* name */
			argv[3]);  /* msg */ 
      status = vsd_process_reply (io);
      break;
    case CMD_VUSER_MSGDEL:
      vsd_send_command (io, NULL, 
			"VUSER_MSGDEL %s %s",
			argv[2],   /* name */
			argv[3]);  /* msg */ 
      status = vsd_process_reply (io);
      break;
    case CMD_VUSER_MSGGET:
      vsd_send_command (io, NULL, 
			"VUSER_MSGGET %s %s",
			argv[2],   /* name */
			argv[3]);  /* msg */ 
      status = vsd_process_reply (io);
      break;
    case CMD_VUSER_MSGSET:
      vsd_send_command_file (io, argv[4], 
			     "VUSER_MSGSET %s %s",
			     argv[2],   /* name */
			     argv[3]);  /* msg */ 
      status = vsd_process_reply (io);
      break;
    case CMD_VUSER_MSGLIST:
      vsd_send_command (io, NULL, 
			"VUSER_MSGLIST %s",
			argv[2]);  /* name */ 
      status = vsd_process_reply (io);
      break;
    default:
      return "mod_vuser: command not recognised";
      break;
    }
  return status;
}

#else  /* ! VSD_LIB */
/* Return 0 if command processed, 1 if not recognised.  */
int mod_vuser_cmd (struct connection *vc,
		   int level, const char *command,
		   const int argc, const char *argv[], const char *data)
{
  int token;
  struct command_tag *tag;

  tag = module_lookup_command (clist, level, command);
  if (tag == NULL)
    return 1;

  token = tag->token;
  switch (token)
    {
    case CMD_VUSER_ADD:
      vuser_add (vc, argc, argv);
      break;
    case CMD_VUSER_DEL:
      vuser_del (vc, argc, argv);
      break;
    case CMD_VUSER_DFLTADD:
      vuser_dfltadd (vc, argc, argv);
      break;
    case CMD_VUSER_DFLTDEL:
      vuser_dfltdel (vc, argc, argv);
      break;
    case CMD_VUSER_FWDADD:
      vuser_fwdadd (vc, argc, argv);
      break;
    case CMD_VUSER_FWDDEL:
      vuser_fwddel (vc, argc, argv);
      break;
    case CMD_VUSER_RSPADD:
      vuser_rspadd (vc, argc, argv);
      break;
    case CMD_VUSER_RSPDEL:
      vuser_rspdel (vc, argc, argv);
      break;
    case CMD_VUSER_LIST:
      vuser_list (vc, argc, argv);
      break;
    case CMD_VUSER_MBXADD:
      vuser_mbxadd (vc, argc, argv);
      break;
    case CMD_VUSER_MBXDEL:
      vuser_mbxdel (vc, argc, argv);
      break;
    case CMD_VUSER_MBXLIST:
      vuser_mbxlist (vc, argc, argv);
      break;
    case CMD_VUSER_MBXPASS:
      vuser_mbxpass (vc, argc, argv);
      break;
    case CMD_VUSER_MBXENABLE:
      vuser_mbxenable (vc, argc, argv);
      break;
    case CMD_VUSER_MBXDISABLE:
      vuser_mbxdisable (vc, argc, argv);
      break;
    case CMD_VUSER_MSGADD:
      vuser_msgadd (vc, argc, argv);
      break;
    case CMD_VUSER_MSGDEL:
      vuser_msgdel (vc, argc, argv);
      break;
    case CMD_VUSER_MSGGET:
      vuser_msgget (vc, argc, argv);
      break;
    case CMD_VUSER_MSGSET:
      vuser_msgset (vc, argc, argv, data);
      break;
    case CMD_VUSER_MSGLIST:
      vuser_msglist (vc, argc, argv);
      break;
    default:
      return 1;
      break;
    }

  return 0;
}
#endif  /* VSD_LIB */

module vuser_module =
{
  mod_vuser_check_cmd,
  mod_vuser_cmd
};




