/* Client SSL encryption layer.
   Copyright (c) 2001 Designer Servers Ltd.
   Written by Nick Burrett <nick@dsvr.net>

   This file is part of the Virtual Server Administrator (FreeVSD)

   FreeVSD is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   FreeVSD is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with FreeVSD; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#ifndef CRYPTSSL_H
#define CRYPTSSL_H

#include <stdlib.h>
#include <openssl/crypto.h>
#include <openssl/x509.h>
#include <openssl/pem.h>
#include <openssl/ssl.h>
#include <openssl/err.h>

class CryptSSL
{
 private:
  SSL *ssl;
  SSL_CTX *ctx;
  char *certificate;
 public:
  int Init (int fd);
  int Destroy (void);
  inline int Read (char *buff, int bytes)
    { return SSL_read (ssl, buff, bytes); }
  inline int Write (const char *buff, int bytes)
    { return SSL_write (ssl, buff, bytes); }
  inline const char *GetCertificate (void) { return certificate; }
  CryptSSL (void) { certificate = NULL; }
  ~CryptSSL (void) { if (certificate) free (certificate); }
};

#endif
