/* In-memory lookup database for virtual server mappings.
   Copyright (c) 2001 Designer Servers Ltd.
   Written by Nick Burrett <nick@dsvr.net>

   This file is part of the Virtual Server Administrator (FreeVSD)

   FreeVSD is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   FreeVSD is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with FreeVSD; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <stdlib.h>
#include <string.h>

#include "lookup.h"


Lookup::Lookup (void)
{
  vs_database = NULL;
  vs_size = vs_entries = 0;
  ns_database = NULL;
  ns_size = ns_entries = 0;
}

Lookup::~Lookup (void)
{
  for (int x = 0; x < vs_size; x++)
    {
      free (vs_database[x].vs);
      free (vs_database[x].svr);
    }
  free (vs_database);
  for (int x = 0; x < ns_size; x++)
    {
      free (ns_database[x].vs);
      free (ns_database[x].pri);
      free (ns_database[x].sec);
    }
  free (ns_database);
}

void Lookup::Add (const char *name, const char *map)
{
  if (vs_database == NULL)
    {
      vs_size = 32;
      vs_database = (struct vs_map *) malloc (sizeof (struct vs_map)
					      * vs_size);
    }
  if (vs_entries == vs_size - 1)
    {
      vs_size += 32;
      vs_database = (struct vs_map *) realloc (vs_database,
					       sizeof (struct vs_map) * vs_size);
    }
  vs_database[vs_entries].vs = strdup (name);
  vs_database[vs_entries].svr = strdup (map);
  vs_entries ++;
}

void Lookup::Add (const char *name, const char *prins, const char *secns)
{
  if (ns_database == NULL)
    {
      ns_size = 32;
      ns_database = (struct ns_map *) malloc (sizeof (struct ns_map)
					      * ns_size);
    }
  if (ns_entries == ns_size - 1)
    {
      ns_size += 32;
      ns_database = (struct ns_map *) realloc (ns_database,
					       sizeof (struct ns_map) * ns_size);
    }
  ns_database[ns_entries].vs = strdup (name);
  ns_database[ns_entries].pri = strdup (prins);
  ns_database[ns_entries].sec = strdup (secns);
  ns_entries ++;
}

const char *Lookup::Find (const char *name)
{
  for (int x = 0; x < vs_entries; x++)
    if (strcmp (name, vs_database[x].vs) == 0)
      return vs_database[x].svr;

  return NULL;
}

int Lookup::Find (const char *name, char *prins, char *secns)
{
  for (int x = 0; x < ns_entries; x++)
    if (strcmp (name, ns_database[x].vs) == 0)
      {
	strcpy (prins, ns_database[x].pri);
	strcpy (secns, ns_database[x].sec);
	return 0;
      }

  return 1;
}
